import React, { createContext, useEffect, useState } from 'react';
import { TagCategoryInfo, TagInfo } from '../api';
import { tagController } from '../controllers';

/**
 * Interface of a tag context provider.
 */
interface ITagCategoryContextProvider {
    /**
     * Tags managed by the context.
     */
    tagCategories: TagCategoryInfo[] | null;

    /**
     * Sets new tags.
     * @param newTags An array of new tags.
     */
    setTagCategories: (newTagCategories: TagCategoryInfo[] | null) => void;
}

/**
 * The tag context that manages available tags.
 */
export const TagCategoryContext = createContext<ITagCategoryContextProvider>({
    /**
     * Default tags.
     */
    tagCategories: null,

    /**
     * Default implementation of setTags method.
     * @param v Array of new tags.
     */
    setTagCategories: (c) => {
        return;
    },
});

/**
 * Provider of the tag context.
 * @param props Children that should have access to the tag context.
 * @returns Prepared html of the provider.
 */
const TagCategoryProvider = (props: { children: React.ReactNode }) => {
    /**
     * Tags managed by the context.
     */
    const [tagCategories, setTagCategories] = useState<TagCategoryInfo[] | null>(null);

    /**
     * Initializes the context.
     */
    useEffect(() => {
        tagController.tagsGet().then((tagTree) => {
            if (typeof tagTree.data.tagCategories != 'undefined') {
                setTagCategories(tagTree.data.tagCategories);
            }
        });
    }, []);

    return (
        <TagCategoryContext.Provider
            value={{
                tagCategories,
                setTagCategories,
            }}
        >
            {props.children}
        </TagCategoryContext.Provider>
    );
};

export default TagCategoryProvider;
